/*
    Unpanel
    GNOME Shell 46+ extension
    Copyright @fthx 2024
    License GPL v3
*/

import GObject from 'gi://GObject';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

const PANEL_FADE_DURATION = 750; // ms


export default class PanelFreeExtension {
    _hasMaximizedWindow() {
        let hasMaximizedWindow = false;

        let activeWorkspace = global.workspace_manager.get_active_workspace()
        let windowsList = activeWorkspace.list_windows();

        for (let window of windowsList) {
            let isMaximizedWindow = window.maximized_vertically;
            hasMaximizedWindow = hasMaximizedWindow || isMaximizedWindow;
        }

        return hasMaximizedWindow;
    }

    _hasVisibleWindow() {
        let hasVisibleWindow = false;

        let activeWorkspace = global.workspace_manager.get_active_workspace()
        let windowsList = activeWorkspace.list_windows();

        for (let window of windowsList) {
            let isVisibleWindow = !window.is_hidden();
            hasVisibleWindow = hasVisibleWindow || isVisibleWindow;
        }

        return hasVisibleWindow;
    }

    _showPanel() {
        Main.panel.height = this._panelHeight;

        for (let box of [Main.panel._leftBox, Main.panel._centerBox, Main.panel._rightBox]) {
            box.ease({
                opacity: 255,
                duration: PANEL_FADE_DURATION,
            });
        }
    }

    _hidePanel() {
        Main.panel.height = 1;

        for (let box of [Main.panel._leftBox, Main.panel._centerBox, Main.panel._rightBox]) {
            box.ease({
                opacity: 0,
                duration: PANEL_FADE_DURATION,
            });
        }
    }

    _trackMaximizedWindow(window) {
        this._triggerPanel();

        window.connectObject(
            'notify::maximized-vertically', this._triggerPanel.bind(this),
            'unmanaging', this._triggerPanel.bind(this),
            'unmanaging', (window) => window.disconnectObject(this), GObject.ConnectFlags.AFTER,
            this);
    }

    _triggerPanel() {
        if (this._hasMaximizedWindow()) {
            this._hidePanel();
        } else {
            this._showPanel();
        }
    }

    _onWorkspaceChanged() {
        if (!Main.overview.visible) {
            this._triggerPanel();
        }
    }

    _onShowDesktopChanged() {
        if (this._hasVisibleWindow()) {
            this._showPanel();
        } else {
            this._triggerPanel();
        }
    }

    enable() {
        this._panelHeight = Main.panel.height;

        global.display.connectObject(
            'window-created', (display, window) => this._trackMaximizedWindow(window),
            this);

        global.workspace_manager.connectObject(
            'active-workspace-changed', this._onWorkspaceChanged.bind(this),
            'showing-desktop-changed', this._onShowDesktopChanged.bind(this),
            this);

        Main.overview.connectObject(
            'showing', this._showPanel.bind(this),
            'hiding', this._triggerPanel.bind(this),
            this);
    }

    disable() {
        global.display.disconnectObject(this);
        global.workspace_manager.disconnectObject(this);
        Main.overview.disconnectObject(this);

        this._showPanel();
        this._panelHeight = null;
    }
}
